package config

import (
	"encoding/json"
	"fmt"
	"io/ioutil"
	"regexp"
	"strings"
    "bytes"
    "errors"
)

type Config struct {
	DB DB `json:"db"`
}

type DB struct {
	Engine   string `json:"engine"`
	User     string `json:"user"`
	Password string `json:"password"`
	Host     string `json:"host"`
	Port     int64  `json:"port"`
	Database string `json:"database"`
	Options  map[string]string
}

var search_dir = "."

func New() (*Config, error) {
	data, err := ioutil.ReadFile(strings.Join([]string{search_dir, "/config.json"}, ""))
	if err != nil {
		return nil, err
	}
	var config Config
	decoder := json.NewDecoder(bytes.NewReader(data))
	decoder.Decode(&config)
	return &config, nil
}

func (self DB) GetURL() (*string, error) {
	if !(CheckWord(self.Engine) && CheckWord(self.User) && CheckWord(self.Password) && CheckWord(self.Database) && CheckHost(self.Host)) {
		return nil, errors.New("Database configuration doesn't pass the constraints.")
	}
	return_string := fmt.Sprintf("%s://%s:%s@%s:%d/%s", self.Engine, self.User, self.Password, self.Host, self.Port, self.Database)
	if self.Options != nil && len(self.Options) > 0 {
		return_string = strings.Join([]string{return_string, "?"}, "")
		for key, value := range self.Options {
			return_string = strings.Join([]string{
				return_string,
				strings.Join([]string{key, value}, "="),
			}, "&")
		}
	}
	return &return_string, nil
}

func CheckHost(word string) bool {
	match, err := regexp.MatchString("^(\\w|\\.)+$", word)
	if err != nil {
		fmt.Println(err.Error())
		return false
	}
	return match
}

func CheckWord(word string) bool {
	match, err := regexp.MatchString("^\\w+$", word)
	if err != nil {
		fmt.Println(err.Error())
		return false
	}
	return match
}
