package groupmanager

import (
	"WWWShop/wwwshop/model/group"
	"database/sql"
	"errors"
	"fmt"
	"github.com/lib/pq/hstore"
	"log"
	"strings"
)

type GroupManager struct {
	db *sql.DB
}

func New(db *sql.DB) GroupManager {
	return GroupManager{db}
}

func (self GroupManager) Add(groupname string, permissions map[string]string) (*group.Group, error) {
	var id int64
	if permissions == nil {
		permissions = make(map[string]string)
	}
	permissions_null := hstore.Hstore{
		Map: make(map[string]sql.NullString),
	}
	for key, value := range permissions {
		permissions_null.Map[key] = sql.NullString{
			String: value,
			Valid:  true,
		}
	}
	result := self.db.QueryRow("INSERT INTO groups(groupname, permissions) VALUES ($1, $2) RETURNING id", groupname, permissions_null)
	err := result.Scan(&id)
	if err != nil {
		return nil, err
	}
	group := group.New(id, groupname, permissions)
	return &group, nil
}

type QueryParameter interface{}

func (self GroupManager) GenerateWhere(cur_key *int64, ids []int64, groupnames []string) string {
	where_string := "WHERE false "
	if groupnames != nil && len(groupnames) > 0 {
		where_string = strings.Join([]string{where_string, "OR ( false"}, "")
		for range groupnames {
			where_string = strings.Join([]string{where_string, fmt.Sprintf(" OR groupname = $%d", *cur_key)}, "")
			(*cur_key)++
		}
		where_string = strings.Join([]string{where_string, " ) "}, "")
	}
	if ids != nil && len(ids) > 0 {
		where_string = strings.Join([]string{where_string, "OR ( false"}, "")
		for range ids {
			where_string = strings.Join([]string{where_string, fmt.Sprintf(" OR id = $%d", *cur_key)}, "")
			(*cur_key)++
		}
		where_string = strings.Join([]string{where_string, " ) "}, "")
	}
	return where_string
}

func (self GroupManager) Update(group group.Group) (*group.Group, error) {
	if group.Id == nil && group.Groupname == nil {
		return nil, errors.New("Both Id and Groupname cannot be null on the same time on UPDATE.")
	}
	if group.Permissions == nil {
		return nil, errors.New("Should be at least a field for update .")
	}
	query := "UPDATE groups SET permissions = $1 WHERE true "
	parameters := make([]interface{}, 0)
	permissions_null := hstore.Hstore{
		Map: make(map[string]sql.NullString),
	}
	for key, value := range group.Permissions {
		log.Println(value)
		permissions_null.Map[key] = sql.NullString{
			String: value,
			Valid:  true,
		}
	}
	parameters = append(parameters, permissions_null)
	if group.Id != nil {
		query = strings.Join([]string{query, "AND id = $2 "}, "")
		parameters = append(parameters, *(group.Id))
	} else {
		query = strings.Join([]string{query, "AND groupname = $2 "}, "")

		parameters = append(parameters, *(group.Groupname))
	}
	query = strings.Join([]string{query, " RETURNING id"}, "")
	var id int64
	result := self.db.QueryRow(query, parameters...)
	err := result.Scan(&id)
	if err != nil {
		return nil, err
	}
	groups, err := self.Retrieve(0, []int64{id}, nil)
	group = groups[0]
	if err != nil {
		return nil, err
	}
	return &group, nil
}

func (self GroupManager) Delete(ids []int64, groupnames []string) ([]string, error) {
	var where bool
	var cur_key int64
	var where_string string
	cur_key = 1

	if ids != nil && len(ids) > 0 {
		where = true
	}
	if groupnames != nil && len(groupnames) > 0 {
		where = true
	}

	if where {
		where_string = self.GenerateWhere(&cur_key, ids, groupnames)
	} else {
		return nil, errors.New("Cannot delete without a clause")
	}

	query_parameters := make([]interface{}, 0)
	for _, groupname := range groupnames {
		query_parameters = append(query_parameters, groupname)
	}
	for _, id := range ids {
		query_parameters = append(query_parameters, id)
	}
	query_string := fmt.Sprintf("DELETE FROM groups %s RETURNING groupname", where_string)
	result, err := self.db.Query(
		query_string,
		query_parameters...,
	)
	if err != nil {
		return nil, err
	}
	defer result.Close()
	var deleted_groups []string
	for result.Next() {
		var groupname string
		err := result.Scan(&groupname)
		if err != nil {
			return nil, err
		}
		deleted_groups = append(deleted_groups, groupname)
	}
	if deleted_groups == nil {
		deleted_groups = make([]string, 0)
	}
	return deleted_groups, nil
}

func (self GroupManager) Retrieve(page int64, ids []int64, groupnames []string) ([]group.Group, error) {
	var where bool
	var cur_key int64

	if ids != nil && len(ids) > 0 {
		where = true
	}
	if groupnames != nil && len(groupnames) > 0 {
		where = true
	}
	cur_key = 1
	where_string := ""
	if where {
		where_string = self.GenerateWhere(&cur_key, ids, groupnames)
	}
	page = (10 * page)
	page_string := fmt.Sprintf("LIMIT 10 OFFSET %d", page)
	query_parameters := make([]interface{}, 0)
	for _, groupname := range groupnames {
		query_parameters = append(query_parameters, groupname)
	}
	for _, id := range ids {
		query_parameters = append(query_parameters, id)
	}
	query_string := fmt.Sprintf("SELECT id, groupname, permissions from groups %s %s;", where_string, page_string)
	result, err := self.db.Query(
		query_string,
		query_parameters...,
	)
	if err != nil {
		return nil, err
	}
	defer result.Close()
	var groups []group.Group
	for result.Next() {
		var (
			id_group               int64
			groupname_group        string
			permissions_group_null hstore.Hstore
		)
		err := result.Scan(&id_group, &groupname_group, &permissions_group_null)
		permissions_group := make(map[string]string)
		for key, permission_null := range permissions_group_null.Map {
			if permission_null.Valid {
				permissions_group[key] = permission_null.String
			}
		}
		if err != nil {
			return nil, err
		}
		u := group.New(id_group, groupname_group, permissions_group)
		groups = append(groups, u)
	}
	if groups == nil {
		groups = make([]group.Group, 0)
	}
	return groups, nil
}
